/** Put here CSS that should apply to both public and editor screens. **/

#editor-bar {
  /* We want the edit bar directly below the main navbar. */
  margin-top: -20px;
}

/* workflow states */
.state-public {
  color: #468847;
}

.state-private {
  color: #B94A48;
}
